package org.bfkj.protocol;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.bfkj.utils.LogUtils;
import org.bfkj.utils.MapTools;
import org.springframework.util.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.util.*;
import java.util.zip.GZIPInputStream;

/*es协议*/
public class FtpHandler {

    private static final int BUFFER_SIZE = 4096;

    /**
     * 设置ftp客户端参数并创建ftp客户端
     *
     * @param config ftp连接配置信息
     * @return ftp客户端
     */
    private FTPClient createFtpClient(Map<String, Object> config) throws Exception {
        FTPClient ftp = new FTPClient();
        ftp.connect(Objects.toString(config.get("host")), Objects.nonNull(config.get("port")) ? Integer.parseInt(Objects.toString(config.get("port"))) : 21);
        //common-net的ftpclient默认是使用ASCII_FILE_TYPE，文件会经过ASCII编码转换，所以可能会造成文件损坏
        ftp.login(Objects.toString(config.get("username")), Objects.toString(config.get("password")));//登录
        ftp.setConnectTimeout(Objects.nonNull(config.get("timeOut")) ? Integer.parseInt(Objects.toString(config.get("timeOut"))) : (1000 * 120)); //120秒
        ftp.setDataTimeout(Duration.ofSeconds(Objects.nonNull(config.get("timeOut")) ? Integer.parseInt(Objects.toString(config.get("timeOut"))) : (1000 * 120))); //120秒
        ftp.setBufferSize(1024 * 1024 * 10); //每次读取文件流时缓存数组的大小
        ftp.setRemoteVerificationEnabled(false); // 取消服务器获取自身Ip地址和提交的host进行匹配
        ftp.setControlEncoding(Objects.nonNull(config.get("encoding")) ? config.get("encoding").toString() : "UTF-8"); //设置ftp编码
        ftp.setFileType(FTPClient.BINARY_FILE_TYPE);//设置文件编码类型为二进制文件，
        return ftp;
    }

    /**
     *
     * @param inFileName 组装好的文件名称
     * @param connectConfig 连接fpt的连接信息
     * @return
     */
    public Map<String, Object> readData(Object inFileName, String connectConfig) {
        Map<String, Object> connectConfigMaps = (Map<String, Object>) MapTools.strToObj(connectConfig);
        FTPClient ftp = null;
        try {
            List<Object> fileList = new ArrayList<>();
            if (inFileName instanceof  String tpStr){
                fileList.addAll(Arrays.stream(tpStr.split(",")).toList());
            }else {
                if (inFileName instanceof List<?> tpList) {
                    fileList.addAll(tpList);
                }
            }
            String basePath = Objects.toString(connectConfigMaps.get("basePath")); //根路径
            ftp = createFtpClient(connectConfigMaps);
            if (!FTPReply.isPositiveCompletion(ftp.getReplyCode())) {
                System.out.println("未连接到FTP，用户名或密码错误。");
                ftp.logout();
                return MapTools.processFail("未连接到FTP，用户名或密码错误。");
            }
            if (StringUtils.hasLength(basePath)) {
                ftp.changeWorkingDirectory(basePath);
            }
            String rootPath = ftp.printWorkingDirectory(); //ftp根目录
            FTPFile[] ftpFiles = ftp.listFiles(rootPath );
            if (ftpFiles.length == 0) {
                System.out.println("没有找到文件");
                ftp.logout();
                return MapTools.processFail("没有找到文件");
            }
            List<File> files = new ArrayList<>();
            for (FTPFile ftpFile : ftpFiles) {
                String tpFileName = ftpFile.getName();
                System.out.println(tpFileName);
                if (fileList.contains(tpFileName)){
                    String path = System.getProperty("user.dir") + File.separator + "ftp" + File.separator + ftpFile.getName();
                    File targetFile = new File(path);
                    files.add(targetFile);
                    try ( FileOutputStream fileOutputStream = new FileOutputStream(targetFile);){
                        ftp.retrieveFile(ftpFile.getName(), fileOutputStream);
                    }catch (Exception e){
                        System.out.println("数据下载异常".concat(LogUtils.getException(e)));
                    }
                }
            }
            ftp.logout();//退出ftp
            List<String> fileContent = new ArrayList<>();
            for (File ftpFile : files) {
                String path = System.getProperty("user.dir") + File.separator + "ftp" + File.separator + ftpFile.getName();
                File targetFile = new File(path);
                if (targetFile.exists()) {
                    try(GZIPInputStream gzipInputStream = new GZIPInputStream(Files.newInputStream(targetFile.toPath()));
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    ) {
                        byte[] buf = new byte[BUFFER_SIZE];
                        int len;
                        while ((len = gzipInputStream.read(buf, 0, BUFFER_SIZE)) != -1) {
                            baos.write(buf, 0, len);
                        }
                        fileContent.add(baos.toString(StandardCharsets.UTF_8));
                    }catch (Exception e){
                        System.out.println("数据解压异常".concat(LogUtils.getException(e)));
                    }
                }
            }
            return MapTools.processSuccess(fileContent);
        } catch (Exception e) {
            return MapTools.processFail("fpt读取文件异常".concat(LogUtils.getException(e)));
        } finally {
            try {
                if (null != ftp) {
                    ftp.disconnect();
                }
            } catch (Exception ex) {
                System.out.println("ftp关闭时异常:".concat(ex.getMessage()));
            }
        }
    }
}
