package org.bfkj.protocol;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.GetResponse;
import org.bfkj.utils.LogUtils;
import org.bfkj.utils.MapTools;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeoutException;

public class MyRabbitMQ {
    private ConnectionFactory connectionFactory;

    /**
     * 采集kafka数据
     *
     * @return 读取内容信息
     */
    public Map<String, Object> readMethod(String sourceObjectName, String connectConfig) {
        if (Objects.isNull(sourceObjectName) || Objects.isNull(connectConfig)) {
            return MapTools.processFail(Objects.isNull(sourceObjectName) ? "队列为空" : "连接字符串为空");
        }

        Map<String, Object>  connectConfigMaps = (Map<String, Object>) MapTools.strToObj(connectConfig);
        try (Connection connection = createConnectFactory(connectConfigMaps).newConnection(); Channel currentChannel = connection.createChannel();) {
            //创建消息通道
            currentChannel.queueDeclare(sourceObjectName, true, false, false, null);
            currentChannel.basicQos(1); //服务端在同一时刻只发送1条数据
            GetResponse getResponse = currentChannel.basicGet(sourceObjectName, true);
            String repBody = null;
            if (null != getResponse) {
                repBody = new String(getResponse.getBody());
            }
            return MapTools.processSuccess(repBody);
        } catch (IOException | TimeoutException e) {
            return MapTools.processFail(LogUtils.getException(e));
        } finally {
            connectionFactory = null;
        }
    }

    public ConnectionFactory createConnectFactory(Map<String, Object> connectConfig) {
        if (null == connectionFactory) {
            connectionFactory = new ConnectionFactory();
            connectionFactory.setHost(connectConfig.get("host").toString());
            connectionFactory.setUsername(connectConfig.get("username").toString());
            connectionFactory.setPassword(connectConfig.get("password").toString());
            connectionFactory.setPort(Integer.parseInt(connectConfig.get("port").toString()));
            Object virtualHost = connectConfig.get("virtualHost");
            if (Objects.nonNull(virtualHost)) connectionFactory.setVirtualHost(virtualHost.toString());
            connectionFactory.setConnectionTimeout(3000);
        }
        return connectionFactory;
    }


    /**
     * 数据发送到kafka
     *
     * @return 发送结果信息
     */
    public Map<String,Object > sendMethod(String sourceObjectName, String routingKey, String data, String connectConfig, String exchangeName) {
        if (Objects.isNull(sourceObjectName) || Objects.isNull(data)) {
            return MapTools.processFail(Objects.isNull(sourceObjectName) ? ("队列为空 " + data) : ("数据内容为空 " + data));
        }
        Map<String, Object>  connectConfigMaps = (Map<String, Object>) MapTools.strToObj(connectConfig);
        ConnectionFactory connectFactory = createConnectFactory(connectConfigMaps);
        try (Connection connection = connectFactory.newConnection()) {
            try (Channel channel = connection.createChannel()) {
                routingKey = (routingKey == null) ? sourceObjectName : routingKey;
                exchangeName = (null == exchangeName) ? "" : exchangeName;
//                AMQP.BasicProperties properties = new AMQP.BasicProperties.Builder().deliveryMode(2).contentEncoding("UTF-8").build();
                //第一个参数是exchange参数，如果是为空字符串，那么就会发送到(AMQP default)默认的exchange，而且routingKey
                //便是所要发送到的队列名
                //todo 如果rabbitmq的服务端为绑定路由，那么写入不进行
                //todo exchangeName为“” 则rabbitmq服务端不需要配置
                //todo exchangeName为不为"" 则rabbitmq必须绑定
                channel.basicPublish(exchangeName, routingKey, null, data.getBytes(StandardCharsets.UTF_8));
            } catch (Exception e) {
                return MapTools.processFail(LogUtils.getException(e));
            }
        } catch (IOException | TimeoutException e) {
            return MapTools.processFail(LogUtils.getException(e));
        } finally {
            connectionFactory = null;
        }
        return MapTools.processSuccess(null);
    }

    /**
     * 关闭kafka信息
     *
     * @throws Exception
     */
    public void close() {
        if (Objects.nonNull(connectionFactory)) {
            connectionFactory = null;
        }
    }
}
