package org.bfkj.utils;

import org.openjdk.nashorn.api.scripting.NashornScriptEngineFactory;
import org.openjdk.nashorn.api.scripting.ScriptObjectMirror;

import javax.script.*;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/***
 *   //如果是java，可以通过事件和数据源自动完成算法配置，实现省略各类协议读写的配置
 *   //数据结构项的使用，可以减少JS的编写，在此处从数据结构项表获取，实现对JAVA算法返回结果的深加工
 */
public class ScriptEnginePro {
    private final Map<Long, Bindings> bindingMap = new HashMap<>(50); //JS执行绑定参数Map
    private String errorMessage = null; //初始化时，对象异常信息

    private Method javaMethod; //java执行函数
    private Method javaMethodClose; //java执行函数
    private Object classInstance;//java执行实例

    private ScriptEngine scriptEngine; //脚本执行引擎
    private CompiledScript scriptCompile;//脚本编译

    private String library_id; //算法ID
    private String library_type; //算法类型： // 1. java 2. js 脚本 3. 数据库  4. webapi
    private String computing_expression; //脚本： 1. java输出执行名称 2.JS脚本 3.sql语句 4.空

    private Map<String, String> parmaNames = new HashMap<>();//脚本参数名称，用于数据处理对象进行参数订阅
    List<Object> javaParams = new ArrayList<>();//java执行参数：因为没有变量名需要对应参数顺序

    private long threadId = Thread.currentThread().getId();//当前引擎线程编号

    public ScriptEnginePro(Map<String, Object> calcInfo) {//引擎构造函数

        try {
            if (Objects.isNull(calcInfo) || Objects.isNull(calcInfo.get("library_type")) || Objects.isNull(calcInfo.get("computing_expression"))) {
                errorMessage = "核心参数为空：".concat(Objects.isNull(calcInfo) ? "" : calcInfo.toString());
                return;
            }
            library_id = calcInfo.get("library_id").toString();//算法编号
            library_type = calcInfo.get("library_type").toString();//算法类型
            if (!"1,2".contains(library_type)) {//目前只支持JS和JAVA
                errorMessage = "未支持的算法类型：".concat(calcInfo.toString());
                return;
            }
            computing_expression = calcInfo.get("computing_expression").toString();//脚本
            if ("1".equals(library_type)) {
                errorMessage = initJava(calcInfo.get("filePath"), calcInfo.get("className"));//初始化java执行对象
                if (MapTools.isBlank(errorMessage)) {
                    return;
                }
            }
            //初始化脚本执行对象
            if ("2".equals(library_type)) { // 1. java时记录方法名 (缺 目标对象字段) 2. js时 记录脚本（后期可以运用数据源中的文件名在加载js文件,此时脚本为方法名）
                ScriptEngineManager SEManager = new ScriptEngineManager();
                NashornScriptEngineFactory NSEFactory = null;

                for (ScriptEngineFactory SEFactory : SEManager.getEngineFactories()) {
                    if ("OpenJDK Nashorn".equalsIgnoreCase(SEFactory.getEngineName())) {
                        NSEFactory = (NashornScriptEngineFactory) SEFactory;
//                        NSEFactory.getScriptEngine("--language=es6");
                        System.setProperty("nashorn.args", "--language=es6");
                        break;
                    }
                }
                if (Objects.isNull(NSEFactory)) {
                    errorMessage = "未找到OpenJDK Nashorn";
                    return;
                }
                scriptEngine = NSEFactory.getScriptEngine("--no-java", "--global-per-engine");//引擎加速：--global-per-engine ,安全：--no-java
                try {
                    initParam();
                    scriptCompile = ((Compilable) scriptEngine).compile(computing_expression);
                } catch (ScriptException e) {
                    errorMessage = "JS脚本预编译时出现错误" + LogUtils.getException(e);
                }
            }
        } catch (Exception e) {
            errorMessage = "引擎创建失败".concat(LogUtils.getException(e));
        }
    }

    //脚本引擎对象关闭时的资源释放
    public void close() {
        //如果算法的类型是数据库 则获取jdbctempalte进行关闭;如果是其他协议则 直接调用invoke方法进行关闭
        if (Objects.nonNull(javaMethodClose)) {
            try {
                javaMethodClose.invoke(classInstance);
            } catch (IllegalAccessException | InvocationTargetException e) {
                System.out.println("JAVA反射关闭方法调用失败: ".concat(Objects.nonNull(classInstance) ? classInstance.toString() : "").concat("释放资源失败"));
            }
        }
    }

    //统一成功信息处理
    public Map<String, Object> processSuccess(Object returnData) {
        if (returnData instanceof Map) {
            Map<String, Object> resultData1 = new HashMap<>();
            try {
                resultData1 = (Map<String, Object>) returnData;
            } catch (Exception e) {
                resultData1.put("returnData", returnData);
            }
            resultData1.put("code", "0");
            return resultData1;
        }
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "0");
        returnMap.put("returnData", returnData);
        return returnMap;
    }

    //统一错误信息处理
    public Map<String, Object> processFail(String errorMessage) {
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "-1");
        returnMap.put("message", errorMessage);
        returnMap.put("library_id", library_id);
        return returnMap;
    }

    //初始化JAVA执行对象
    private String initJava(Object filePath, Object className) {
        if (Objects.isNull(className) || Objects.isNull(computing_expression)) {
            return "java执行参数不全，类名:".concat(Objects.isNull(className) ? "" : className.toString()).concat("; 方法名：").concat(computing_expression);
        }
        if (Objects.nonNull(filePath)) {//文件名不为空则加载外部JAR
            File file = new File(System.getProperty("user.dir").concat(File.separator).concat("plugins").concat(File.separator).concat(filePath.toString()));
            if (!file.exists()) {
                return "外部文件加载不存在：".concat(System.getProperty("user.dir")).concat(File.separator).concat("plugins").concat(File.separator).concat(filePath.toString());
            }
            Method addURLMethod = null;
            boolean accessible = false;
            try {
                addURLMethod = URLClassLoader.class.getDeclaredMethod("addURL", URL.class);
                URLClassLoader classLoader = (URLClassLoader) ClassLoader.getSystemClassLoader();
                accessible = addURLMethod.canAccess(null); //获取方法的访问权限以便恢复:判读是否可以访问: true  代表可以访问
                if (!accessible) { //判读是否可以访问，如果是false，那就必须setAccessible(true)
                    addURLMethod.setAccessible(true);//实际上是为了提效
                }
                addURLMethod.invoke(classLoader, file.toURI().toURL());//加载到系统中
            } catch (Exception e) {
                return "JAR加载到环境失败".concat(LogUtils.getException(e));
            } finally {
                if (addURLMethod != null && !accessible) {
                    addURLMethod.setAccessible(false);//恢复原访问权限
                }
            }
        }
        try {
            initParam(); //初始化JAVA执行参数，只处理参数名，避免编译失败
            Class<?> classExample = Class.forName(className.toString()); //获取类实例
            classInstance = classExample.getConstructor().newInstance();//类实例接口 无参数构造
            for (Method currentMethod : classExample.getMethods()) {//循环所有方法
                String methodName = currentMethod.getName();
                if (methodName.equals(computing_expression.substring(0, computing_expression.indexOf("(")))) {
                    javaMethod = currentMethod;
                }
                if ("close".equals(methodName)) {
                    javaMethodClose = currentMethod;
                }
            }
        } catch (Exception e) {
            return "方法编译异常:".concat(LogUtils.getException(e));
        }
        return null;
    }

    //初始化执行参数名，同时修正表达式
    private void initParam() {
        Pattern regExpression = Pattern.compile("(?<=《)([^》]+)?(?=》)");//书名号方式提取正则表达式
        Matcher varNameList = regExpression.matcher(computing_expression); //提取书名号中的参数名
        int index = 0; //用于记录JAVA参数对应实际运行参数的位置
        while (varNameList.find()) {//循环提取的参数
            String varName = varNameList.group();//变量名String.dataContent
            index++;
            parmaNames.put(varName, "1".equals(library_type) ? String.valueOf(index) : (varName.replaceAll("\\.", "")).concat(String.valueOf(threadId)));//添加到变量列表中
            if ("1".equals(library_type)) {  // java参数
                javaParams.add(null);//先初始化一个JAVA参数,因为没有键名对应，所以只能按顺序对应（数据订阅）
            }
            computing_expression = computing_expression.replaceFirst("《" + varName + "》", "1".equals(library_type) ? "" : parmaNames.get(varName));
        }
    }

    //引擎执行入口函数
    public Map<String, Object> execScript(Map<String, Object> execData) {
        if (null != errorMessage) {
            return processFail(errorMessage);
        }
        if ("1".equals(library_type)) {
            try {
                for (String varName : execData.keySet()) {
                    javaParams.set(Integer.parseInt(parmaNames.get(varName)) - 1, execData.get(varName));
                }
                Object invoke = javaMethod.invoke(classInstance, javaParams.toArray());
                return processSuccess(invoke);//数组方式传参，无需参数名，按顺序对应
            } catch (Exception e) {
                return processFail("JAVA执行失败，执行参数：".concat(javaParams.toString()).concat("；异常信息：").concat(LogUtils.getException(e)));
            }
        }
        if ("2".equals(library_type)) {
            Bindings bindings = bindingMap.computeIfAbsent(threadId, k -> scriptEngine.createBindings());
            bindings.putAll(execData);//JS的参数是MAP，需要参数名对应
            try {
                Object jsReturnData = scriptCompile.eval(bindings);
                if (jsReturnData instanceof ScriptObjectMirror scriptObjec) {
                    recursionEencapsulation(scriptObjec); //处理JS返回MAP时变成了数组 List<Map<String,Object>   // {0:{},1:{}}   [{},{},{}]
                    if (scriptObjec.isArray()) {// -- >  [{},{},{}]  []
                        List<Object> tempList = new ArrayList<>();
                        scriptObjec.forEach((index, o) -> tempList.add(o));
                        jsReturnData = tempList;
                    }
                    jsReturnData = MapTools.strToObj(MapTools.objToJSONStr(jsReturnData));//JS脚本对象转为JAVA数据对象
                }
                return processSuccess(jsReturnData);
            } catch (ScriptException e) {
                return processFail("JS执行失败，".concat("；异常信息：").concat(LogUtils.getException(e)));
            }
        }
        return processFail("未支持的算法类型：");
    }

    //修订JS返回值MAP变数组的问题
    private void recursionEencapsulation(ScriptObjectMirror scriptObject) {
        for (String key : scriptObject.keySet()) { // scriptObject 实际是Map 循环map的所有key--- > {"A","B"}
            Object mirror = scriptObject.get(key); // 获取当前可以的值mirror "B"
            if (mirror instanceof ScriptObjectMirror) {// 如果当前值是map
                recursionEencapsulation((ScriptObjectMirror) mirror); // 继续下一层
                if (isArray((ScriptObjectMirror) mirror)) {//如果当前值是一个数组则 转换成List<Map<String,Object>>
                    List<Object> list = parseArray((ScriptObjectMirror) mirror);
                    scriptObject.put(key, list);
                }
            }
        }
    }

    /**
     * 数组判定
     *
     * @param scriptObject 数据内容
     * @return true、false
     */
    private Boolean isArray(ScriptObjectMirror scriptObject) {
        int index = 0;
        for (String key : scriptObject.keySet()) {
            if (!key.equals(index + "")) {
                return false;
            }
            index++;
        }
        return true;
    }

    /**
     * 对象转数组
     *
     * @param scriptObject 对象
     * @return list数组
     */
    private List<Object> parseArray(ScriptObjectMirror scriptObject) {
        List<Object> arrayList = new ArrayList<Object>();
        for (String key : scriptObject.keySet()) {
            Object obj = scriptObject.get(key);
            arrayList.add(obj);
        }
        return arrayList;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    public Map<String, String> getParmaNames() {
        return parmaNames;
    }
}
